@@ -24,23 +24,26 @@
 #include <kfileitemactionplugin.h>
 #include <KFileItemListProperties>
 
+#include "kded_bluedevil.h"
+
 class QAction;
 class KFileItemListProperties;
 class QWidget;
 
 class SendFileItemAction : public KFileItemActionPlugin
 {
-Q_OBJECT
+    Q_OBJECT
 public:
     SendFileItemAction(QObject* parent, const QVariantList &args);
-    virtual QList< QAction* > actions(const KFileItemListProperties& fileItemInfos, QWidget* parentWidget) const;
+    virtual QList<QAction*> actions(const KFileItemListProperties &fileItemInfos, QWidget *parentWidget) const;
 
 private Q_SLOTS:
     void deviceTriggered();
     void otherTriggered();
 
 private:
+    org::kde::BlueDevil *m_kded;
     KFileItemListProperties m_fileItemInfos;
 };
 
-#endif // SENDFILEITEMACTION_H
\ No newline at end of file
+#endif // SENDFILEITEMACTION_H
