@@ -52,12 +52,15 @@ extern "C" int KDE_EXPORT kdemain(int argc, char **argv)
     return 0;
 }
 
+static bool urlIsRoot(const KUrl &url)
+{
+    return (url.directory() == QLatin1String("/") || url.directory().isEmpty()) && url.fileName().isEmpty();
+}
+
 KioFtp::KioFtp(const QByteArray &pool, const QByteArray &app)
     : SlaveBase("obexftp", pool, app)
     , m_transfer(0)
 {
-    m_settingHost = false;
-
     m_timer = new QTimer();
     m_timer->setInterval(100);
 
@@ -78,6 +81,47 @@ void KioFtp::launchProgressBar()
     m_timer->start();
 }
 
+void KioFtp::connectToHost()
+{
+    QDBusPendingReply<QString> reply = m_kded->session(m_host);
+    reply.waitForFinished();
+
+    const QString &sessionPath = reply.value();
+
+    if (reply.isError() || sessionPath.isEmpty()) {
+        kDebug() << reply.error().message();
+        kDebug() << reply.error().name();
+
+        delete m_transfer;
+        m_transfer = 0;
+        m_sessionPath.clear();
+        return;
+    }
+
+    if (m_sessionPath != sessionPath) {
+        m_statMap.clear();
+        delete m_transfer;
+        m_transfer = new org::bluez::obex::FileTransfer1("org.bluez.obex", sessionPath, QDBusConnection::sessionBus());
+        m_sessionPath = sessionPath;
+    }
+}
+
+bool KioFtp::testConnection()
+{
+    if (!m_kded->isOnline().value()) {
+        error(KIO::ERR_SLAVE_DEFINED, i18n("Obexd service is not running."));
+        return false;
+    }
+
+    connectToHost();
+
+    if (!m_transfer) {
+        error(KIO::ERR_COULD_NOT_CONNECT, m_host);
+        return false;
+    }
+    return true;
+}
+
 void KioFtp::updateProcess()
 {
     if (m_counter == 49) {
@@ -92,35 +136,27 @@ void KioFtp::updateProcess()
 
 void KioFtp::listDir(const KUrl &url)
 {
+    if (!testConnection()) {
+        return;
+    }
+
     kDebug() << "listdir: " << url;
 
     infoMessage(i18n("Retrieving information from remote device..."));
 
-    kDebug() << "Asking for listFolder";
+    kDebug() << "Asking for listFolder" << url.path();
 
-    //TODO: Check if changeFolder fails
-    m_transfer->ChangeFolder(url.path()).waitForFinished();
-
-    QDBusPendingReply <QVariantMapList > reply = m_transfer->ListFolder();
-    reply.waitForFinished();
-
-    if (reply.isError()) {
-        kDebug() << reply.error().message();
-        error(KIO::ERR_SLAVE_DEFINED, i18n("Bluetooth is not enabled"));
-        finished();
+    if (!changeFolder(url.path())) {
         return;
     }
-    QVariantMapList folderList = reply.value();
-    Q_FOREACH(const QVariantMap folder, folderList) {
-        KIO::UDSEntry entry = entryFromInfo(folder);
 
-        KUrl statUrl(url);
-        statUrl.setFileName(folder["Name"].toString());
-        if (!m_statMap.contains(statUrl.prettyUrl())) {
-            kDebug() << "Stat: " << statUrl.prettyUrl() << entry.numberValue(KIO::UDSEntry::UDS_SIZE);
-            m_statMap.insert(statUrl.prettyUrl(), entry);
-        }
+    bool ok;
+    const QList<KIO::UDSEntry> &list = listFolder(url, &ok);
+    if (!ok) {
+        return;
+    }
 
+    Q_FOREACH (const KIO::UDSEntry &entry, list) {
         listEntry(entry, false);
     }
 
@@ -133,11 +169,13 @@ void KioFtp::copy(const KUrl &src, const KUrl &dest, int permissions, KIO::JobFl
     Q_UNUSED(permissions)
     Q_UNUSED(flags)
 
+    if (!testConnection()) {
+        return;
+    }
+
     kDebug() << "copy: " << src.url() << " to " << dest.url();
 
     copyHelper(src, dest);
-
-    finished();
 }
 
 void KioFtp::rename(const KUrl& src, const KUrl& dest, KIO::JobFlags flags)
@@ -146,15 +184,18 @@ void KioFtp::rename(const KUrl& src, const KUrl& dest, KIO::JobFlags flags)
     Q_UNUSED(dest)
     Q_UNUSED(flags)
 
-    error(KIO::ERR_UNSUPPORTED_ACTION, src.prettyUrl());
-    finished();
+    error(KIO::ERR_UNSUPPORTED_ACTION, QString());
 }
 
 void KioFtp::get(const KUrl& url)
 {
+    if (!testConnection()) {
+        return;
+    }
+
     KTemporaryFile tempFile;
     tempFile.setSuffix(url.fileName());
-    tempFile.open();//Create the file
+    tempFile.open(); // Create the file
     kDebug() << tempFile.fileName();
 
     copyHelper(url, KUrl(tempFile.fileName()));
@@ -172,6 +213,10 @@ void KioFtp::get(const KUrl& url)
     finished();
 }
 
+bool KioFtp::cancelTransfer(const QString &transfer)
+{
+    return m_kded->cancelTransfer(transfer);
+}
 
 void KioFtp::setHost(const QString &host, quint16 port, const QString &user, const QString &pass)
 {
@@ -179,35 +224,27 @@ void KioFtp::setHost(const QString &host, quint16 port, const QString &user, con
     Q_UNUSED(user)
     Q_UNUSED(pass)
 
+    m_host = host;
+
     infoMessage(i18n("Connecting to the device"));
 
-    kDebug() << "setHost: " << host;
-
-    kDebug() << "Waiting to stablish the connection 2";
-    QDBusPendingReply <QString > reply = m_kded->session(host);
-    reply.waitForFinished();
-
-    kDebug() << "AFTER" << reply.isError();
-    if (reply.isError()) {
-        kDebug() << reply.error().message();
-        kDebug() << reply.error().name();
-    }
-
-    kDebug() << "Got a path" << reply.value();
-
-    m_address = host;
-    m_sessionPath = reply.value();
-    m_transfer = new org::bluez::obex::FileTransfer1("org.bluez.obex", m_sessionPath, QDBusConnection::sessionBus());
-    m_statMap.clear();
+    connectToHost();
 }
 
 void KioFtp::del(const KUrl& url, bool isfile)
 {
     Q_UNUSED(isfile)
 
-    kDebug() << "Del: " << url.url();
-    m_transfer->ChangeFolder(url.directory()).waitForFinished();
-    m_transfer->Delete(url.fileName()).waitForFinished();
+    if (!testConnection()) {
+        return;
+    }
+    if (!changeFolder(url.directory())) {
+        return;
+    }
+    if (!deleteFile(url.fileName())) {
+        return;
+    }
+
     finished();
 }
 
@@ -215,14 +252,28 @@ void KioFtp::mkdir(const KUrl& url, int permissions)
 {
     Q_UNUSED(permissions)
 
+    if (!testConnection()) {
+        return;
+    }
+
     kDebug() << "MkDir: " << url.url();
-    m_transfer->ChangeFolder(url.directory()).waitForFinished();
-    m_transfer->CreateFolder(url.fileName()).waitForFinished();
+
+    if (!changeFolder(url.directory())) {
+        return;
+    }
+    if (!createFolder(url.fileName())) {
+        return;
+    }
+
     finished();
 }
 
 void KioFtp::stat(const KUrl &url)
 {
+    if (!testConnection()) {
+        return;
+    }
+
     kDebug() << "Stat: " << url.url();
     kDebug() << "Stat Dir: " << url.directory();
     kDebug() << "Stat File: " << url.fileName();
@@ -237,8 +288,7 @@ void KioFtp::stat(const KUrl &url)
 void KioFtp::copyHelper(const KUrl& src, const KUrl& dest)
 {
     if (src.scheme() == "obexftp" && dest.scheme() == "obexftp") {
-        error(KIO::ERR_UNSUPPORTED_ACTION, src.prettyUrl());
-        //TODO: with obexd this seems possible, we should at least try
+        copyWithinObexftp(src, dest);
         return;
     }
 
@@ -253,6 +303,16 @@ void KioFtp::copyHelper(const KUrl& src, const KUrl& dest)
     }
 
     kDebug() << "This shouldn't happen...";
+}
+
+void KioFtp::copyWithinObexftp(const KUrl &src, const KUrl &dest)
+{
+    kDebug() << "Source: " << src << "Dest:" << dest;
+
+    if (!copyFile(src.path(), dest.path())) {
+        return;
+    }
+
     finished();
 }
 
@@ -260,28 +320,17 @@ void KioFtp::copyFromObexftp(const KUrl& src, const KUrl& dest)
 {
     kDebug() << "Source: " << src << "Dest:" << dest;
 
-    //Just in case the url is not in the stat, some times happens...
-    if (!m_statMap.contains(src.prettyUrl())) {
-        kDebug() << "The url is not in the cache, stating it";
-        statHelper(src);
-    }
-
-    if (m_statMap.value(src.prettyUrl()).isDir()) {
-        kDebug() << "Skipping to copy: " << src.prettyUrl();
-        //TODO: Check if dir copying works with obexd
-        error(KIO::ERR_IS_DIRECTORY, src.prettyUrl());
+    if (!changeFolder(src.directory())) {
         return;
     }
 
-    kDebug() << "Changing dir:" << src.directory();
-    m_transfer->ChangeFolder(src.directory()).waitForFinished();
-
     QString dbusPath = m_transfer->GetFile(dest.path(), src.fileName()).value().path();
     kDebug() << "Path from GetFile:" << dbusPath;
 
     int size = m_statMap[src.prettyUrl()].numberValue(KIO::UDSEntry::UDS_SIZE);
+    totalSize(size);
+
     TransferFileJob *getFile = new TransferFileJob(dbusPath, this);
-    getFile->setSize(size);
     getFile->exec();
 
     finished();
@@ -291,14 +340,17 @@ void KioFtp::copyToObexftp(const KUrl& src, const KUrl& dest)
 {
     kDebug() << "Source:" << src << "Dest:" << dest;
 
-    kDebug() << "Changing folder: " << dest.directory();
-    m_transfer->ChangeFolder(dest.directory());
+    if (!changeFolder(dest.directory())) {
+        return;
+    }
+
     QString dbusPath = m_transfer->PutFile(src.path(), dest.fileName()).value().path();
     kDebug() << "Path from PutFile: " << dbusPath;
 
-    QFile file(src.path());
+    int size = QFile(src.path()).size();
+    totalSize(size);
+
     TransferFileJob *putFile = new TransferFileJob(dbusPath, this);
-    putFile->setSize(file.size());
     putFile->exec();
 
     finished();
@@ -314,10 +366,10 @@ void KioFtp::statHelper(const KUrl& url)
         return;
     }
 
-    if ((url.directory() == "/" || url.directory().isEmpty()) && url.fileName().isEmpty()) {
+    if (urlIsRoot(url)) {
         kDebug() << "Url is root";
         KIO::UDSEntry entry;
-        entry.insert(KIO::UDSEntry::UDS_NAME, QString::fromLatin1("/"));
+        entry.insert(KIO::UDSEntry::UDS_NAME, QLatin1String("/"));
         entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
         entry.insert(KIO::UDSEntry::UDS_ACCESS, 0700);
         entry.insert( KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1( "inode/directory" ) );
@@ -325,52 +377,146 @@ void KioFtp::statHelper(const KUrl& url)
         kDebug() << "Adding stat cached: " << url.prettyUrl();
         m_statMap[url.prettyUrl()] = entry;
         statEntry(entry);
-
         return;
     }
 
     kDebug() << "statMap does NOT contains the url";
-    //TODO: Check if changeFolder fails
-    m_transfer->ChangeFolder(url.directory()).waitForFinished();
-    QVariantMapList folderList = m_transfer->ListFolder().value();
-    kDebug() << url.directory() << folderList.count();
-    Q_FOREACH(const QVariantMap folder, folderList) {
-        KIO::UDSEntry entry = entryFromInfo(folder);
 
-        QString fileName = folder["Name"].toString();
-        if (url.fileName() == fileName) {
-            statEntry(entry);
-        }
+    if (!changeFolder(url.directory())) {
+        return;
+    }
 
-        //Most probably the client of the kio will stat each file
-        //so since we are on it, let's cache all of them.
-        KUrl statUrl(url);
-        statUrl.setFileName(fileName);
-        if (!m_statMap.contains(statUrl.prettyUrl())) {
-            kDebug() << "Stat: " << statUrl.prettyUrl() << entry.stringValue(KIO::UDSEntry::UDS_NAME) <<  entry.numberValue(KIO::UDSEntry::UDS_SIZE);
-            m_statMap.insert(statUrl.prettyUrl(), entry);
-        }
+    bool ok;
+    const QList<KIO::UDSEntry> &list = listFolder(url, &ok);
+    if (!ok) {
+        return;
+    }
+
+    Q_FOREACH (const KIO::UDSEntry &entry, list) {
+        statEntry(entry);
     }
 
     kDebug() << "Finished";
 }
 
-KIO::UDSEntry KioFtp::entryFromInfo(const QVariantMap& info)
+QList<KIO::UDSEntry> KioFtp::listFolder(const KUrl &url, bool *ok)
 {
-    kDebug() << info;
+    QList<KIO::UDSEntry> list;
 
-    KIO::UDSEntry entry;
-    entry.insert(KIO::UDSEntry::UDS_NAME, info["Name"].toString());
-    entry.insert(KIO::UDSEntry::UDS_CREATION_TIME, info["Created"].toString());
-    entry.insert(KIO::UDSEntry::UDS_ACCESS, 0700);
-    entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, info["Modified"].toString());
+    QDBusPendingReply<QVariantMapList> reply = m_transfer->ListFolder();
+    reply.waitForFinished();
 
-    if (info["Type"].toString() == QLatin1String("folder")) {
-        entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-    } else {
-        entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
-        entry.insert(KIO::UDSEntry::UDS_SIZE, info["Size"].toLongLong());
+    if (reply.isError()) {
+        error(KIO::ERR_CANNOT_OPEN_FOR_READING, url.directory());
+        *ok = false;
+        return list;
     }
 
-    return entry;
-}
\ No newline at end of file
+    Q_FOREACH (const QVariantMap &item, reply.value()) {
+        KIO::UDSEntry entry;
+        entry.insert(KIO::UDSEntry::UDS_NAME, item["Name"].toString());
+        entry.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, item["Label"].toString());
+        entry.insert(KIO::UDSEntry::UDS_ACCESS, 0700);
+        entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, QDateTime::fromString(item["Modified"].toString(), "yyyyMMddThhmmssZ").toTime_t());
+        entry.insert(KIO::UDSEntry::UDS_SIZE, item["Size"].toLongLong());
+        if (item["Type"] == QLatin1String("folder")) {
+            entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+            entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, "inode/directory");
+        } else {
+            entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
+        }
+        if (urlIsRoot(url)) {
+            updateRootEntryIcon(entry, item["Mem-type"].toString());
+        }
+        list.append(entry);
+
+        // Most probably the client of the kio will stat each file
+        // so since we are on it, let's cache all of them.
+        KUrl statUrl(url);
+        statUrl.setFileName(item["Name"].toString());
+        if (!m_statMap.contains(statUrl.prettyUrl())) {
+            kDebug() << "Stat: " << statUrl.prettyUrl() << entry.stringValue(KIO::UDSEntry::UDS_NAME) << entry.numberValue(KIO::UDSEntry::UDS_SIZE);
+            m_statMap.insert(statUrl.prettyUrl(), entry);
+        }
+    }
+
+    *ok = true;
+    return list;
+}
+
+bool KioFtp::changeFolder(const QString &folder)
+{
+    QDBusPendingReply<> reply = m_transfer->ChangeFolder(folder);
+    reply.waitForFinished();
+
+    if (reply.isError()) {
+        error(KIO::ERR_CANNOT_ENTER_DIRECTORY, folder);
+        return false;
+    }
+    return true;
+}
+
+bool KioFtp::createFolder(const QString &folder)
+{
+    QDBusPendingReply<> reply = m_transfer->CreateFolder(folder);
+    reply.waitForFinished();
+
+    if (reply.isError()) {
+        error(KIO::ERR_COULD_NOT_MKDIR, folder);
+        return false;
+    }
+    return true;
+}
+
+bool KioFtp::copyFile(const QString &src, const QString &dest)
+{
+    QDBusPendingReply<> reply = m_transfer->CopyFile(src, dest);
+    reply.waitForFinished();
+
+    if (reply.isError()) {
+        kDebug() << reply.error().message();
+        // Copying files within obexftp is currently not implemented in obexd
+        if (reply.error().message() == QLatin1String("Not Implemented")) {
+            error(KIO::ERR_UNSUPPORTED_ACTION, src);
+        } else {
+            error(KIO::ERR_COULD_NOT_WRITE, src);
+        }
+        return false;
+    }
+    return true;
+}
+
+bool KioFtp::deleteFile(const QString &file)
+{
+    QDBusPendingReply<> reply = m_transfer->Delete(file);
+    reply.waitForFinished();
+
+    if (reply.isError()) {
+        error(KIO::ERR_CANNOT_DELETE, file);
+        return false;
+    }
+    return true;
+}
+
+void KioFtp::updateRootEntryIcon(KIO::UDSEntry &entry, const QString &memoryType)
+{
+    const QString &path = entry.stringValue(KIO::UDSEntry::UDS_NAME);
+
+    // Nokia (mount-points are C: D: E: ...)
+    if (path.size() == 2 && path.at(1) == QLatin1Char(':')) {
+        if (memoryType.startsWith(QLatin1String("DEV"))) {
+            entry.insert(KIO::UDSEntry::UDS_ICON_NAME, "drive-removable-media");
+        } else if (memoryType == QLatin1String("MMC")) {
+            entry.insert(KIO::UDSEntry::UDS_ICON_NAME, "media-flash-sd-mmc");
+        }
+    }
+    // Android
+    if (entry.stringValue(KIO::UDSEntry::UDS_NAME) == QLatin1String("PHONE_MEMORY")) {
+        entry.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, i18n("Phone memory"));
+        entry.insert(KIO::UDSEntry::UDS_ICON_NAME, "smartphone");
+    } else if (entry.stringValue(KIO::UDSEntry::UDS_NAME) == QLatin1String("EXTERNAL_MEMORY")) {
+        entry.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, i18n("External memory"));
+        entry.insert(KIO::UDSEntry::UDS_ICON_NAME, "media-flash-sd-mmc");
+    }
+}
+
